/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.query;

import java.io.File;
import java.sql.Connection;

import org.databene.commons.StringUtil;
import org.databene.commons.iterator.TableRowIterator;
import org.databene.commons.math.Intervals;
import org.databene.commons.version.VersionNumber;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.jdbacl.ArrayResultSetIterator;

/**
 * Performs a check based on a SQL query that lists defective database rows.<br/><br/>
 * Created: 02.12.2010 16:54:35
 * @since 0.4
 * @author Volker Bergmann
 */
public class DefectQueryCheck extends SanityCheck {

	protected String query;

	public DefectQueryCheck(String name, String query, String table,
			Intervals<VersionNumber> intervalCollection, String defectType, 
			File sourceFile, File tempFolder, File reportFolder, File docPage) {
		super(name, table, intervalCollection, defectType, sourceFile, tempFolder, reportFolder, docPage);
	    this.query = query;
	}
	
	public String getQuery() {
		return query;
	}
	
	public void setQuery(String query) {
		this.query = query;
	}

	@Override
	public String getDetails() {
		return query;
	}
	
	@Override
	public TableRowIterator query(Connection connection) {
		if (StringUtil.isEmpty(query))
			throw new IllegalArgumentException("No query specified for " + getClass().getSimpleName() + "'" + name + "'");
		try {
	    	ArrayResultSetIterator iterator = new ArrayResultSetIterator(connection, query);
			this.verdict = (iterator.hasNext() ? SanityCheckVerdict.FAILED : SanityCheckVerdict.PASSED);
			return new Proxy(iterator);
		} catch (RuntimeException e) {
			this.verdict = SanityCheckVerdict.ERROR;
			throw e;
		}
	}

}
