/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.version.Versions;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.db.DbsTable;
import org.databene.dbsanity.model.query.DefectQueryCheck;
import org.w3c.dom.Element;

/**
 * Parses a &lt;check&gt; element.<br/><br/>
 * Created: 05.12.2010 14:20:22
 * @since 0.4
 * @author Volker Bergmann
 */
public class CheckParser extends DbSanityXMLParser {

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet("name");
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet(
			"table", "defectType", "description", "versions", "requirement", "author", "tags");

	public CheckParser() {
		super("check", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES);
	}

	@Override
	public boolean supports(Element element, Object[] parentPath) {
		String name = element.getNodeName();
		if (!"check".equals(name) && !"test".equals(name))
			return false;
		Object parent = parent(parentPath);
		return (parent instanceof SanityCheckFile || parent instanceof DbsTable);
	}
	
	@Override
	public SanityCheck parse(Element checkElement, Object[] parentPath, DbSanityParseContext context) {
	    SanityCheckFile parentSanityCheckFile = ParseUtil.getParentSanityCheckFile(parentPath);
	    DefectQueryCheck check = new DefectQueryCheck(
	    		XMLUtil.getAttribute(checkElement, "name", true),
	    		null,
	    		XMLUtil.getAttribute(checkElement, "table", false),
	    		Versions.valueOf(XMLUtil.getAttribute(checkElement, "versions", false)),
	    		XMLUtil.getAttribute(checkElement, "defectType", false),
	    		parentSanityCheckFile.getSource(),
	    		parentSanityCheckFile.getTempFolder(),
	    		parentSanityCheckFile.getReportFolder(), 
	    		parentSanityCheckFile.getDocPage());
	    // requirement id
	    check.setRequirement(XMLUtil.getAttribute(checkElement, "requirement", false));
	    context.checkUniqueness(check.getName());
	    String author = getOptionalAttribute("author", checkElement);
	    if (author != null)
	    	check.setAuthor(author);
	    String tags = getOptionalAttribute("tags", checkElement);
	    if (tags != null)
	    	check.setTags(CollectionUtil.toSet(StringUtil.trimAll(tags.split(","))));
	    Object parent = parent(parentPath);
	    if (parent instanceof DbsTable)
	    	check.setTable(((DbsTable) parent).getName());
	    parentSanityCheckFile.addCheck(check);
	    Object[] currentPath = context.createSubPath(parentPath, check);
	    context.parseChildElementsOf(checkElement, currentPath);
	    return check;
	}

}
