/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.SystemInfo;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.query.DefectQueryCheck;
import org.databene.jdbacl.SQLUtil;
import org.w3c.dom.Element;

/**
 * Parses the &lt;foreignKey&gt; part of a &lt;check&gt; and creates a SQL defect query.<br/><br/>
 * Created: 26.05.2011 11:39:27
 * @since 0.8
 * @author Volker Bergmann
 */
public class ForeignKeyParser extends DbSanityXMLParser {
	
	private static final String DEFAULT_DEFECT_TYPE = "foreign-key";

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet(
			"refererColumns", "refereeTable", "refereeColumns");

	public ForeignKeyParser() {
		super("foreignKey", REQUIRED_ATTRIBUTES, null, DefectQueryCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
	    DefectQueryCheck parent = (DefectQueryCheck) this.parent(parentPath);
		String refererTable = parent.getTable();
		if (StringUtil.isEmpty(refererTable))
			throw new SyntaxError("Missing 'table' spec in outer <check>", XMLUtil.format(element));

		// parse XML element
		String[] refererColumns = getRequiredAttribute("refererColumns", element).split(",");
		StringUtil.trimAll(refererColumns);
		String refereeTable = getRequiredAttribute("refereeTable", element);
		String[] refereeColumns = getRequiredAttribute("refereeColumns", element).split(",");
		StringUtil.trimAll(refereeColumns);
		
		// create query
	    String LF = SystemInfo.getLineSeparator();
		String defectQuery = "select * from " + refererTable + " referer " + LF + 
			'\t' + SQLUtil.leftJoin("referer", refererColumns, refereeTable, "referee", refereeColumns) + LF + 
			"\twhere referer." + refererColumns[0] + " is not null " + LF +
			"\tand referee." + refereeColumns[0] + " is null";
		parent.setQuery(defectQuery);
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
	    return defectQuery;
	}

}
